// Reading xyz files:
// File format:
// First line: number of atoms (int)
// Second line: empty
// Following lines: atom symbol (string) x y z coordinates (double)
// Last three lines: metric tensor: three elements per line (double)

#include <iostream> // For generic I/O facilities
#include <fstream>  // For file handling 
#include <iomanip>  // For other I/O manipulators
#include <string>
#include <cstdlib>  // To get exit function
#include "atom.h"

using namespace std;

int main() {
    string filename;
    cout << "Enter an xyz file name: ";
    cin >> filename; 

    ifstream in(filename.c_str()); // ifstream constructor expects const char *
                                   // as its filename argument so C++ strings
                                   // must be converted through c_str()
    if ( in.fail() ) {
        cerr << "Cannot open input file " << endl;
        exit(EXIT_FAILURE);
    }

    int natoms;
    in >> natoms;
    if ( cin.fail() ) {
        cerr << "Bad file format" << endl;
        exit(EXIT_FAILURE);
    } else if ( natoms < 1 ) {
        cerr << "Number of atoms should be > 0!" << endl;
        exit(EXIT_FAILURE);
    }
        
    Atom * atoms;
    try {
        atoms = new Atom[natoms];
    }
    catch (bad_alloc&) {
        cerr << "Cannot allocate " << natoms << " elements" << endl;
    }

    int i;
    for ( i = 0; i < natoms && !in.fail(); i++ ) {
        in >> atoms[i];
    }

    if ( in.fail() ) {
        cerr << "End of file: Expected " << natoms << " atoms, read " << i-1 << endl;
        exit(EXIT_FAILURE);
    }

    int j;
    double metrictensor[3][3];
    for ( i = 0; i < 3 && !in.fail(); i++ )
        for ( j = 0; j < 3 && !in.fail(); j++ )
            in >> metrictensor[i][j];

    if ( in.fail() ) {
        cerr << "Too few metric tensor elements read, expected 9, read "
             << (i-1)*3 + j-1 << endl;
        exit(EXIT_FAILURE);
    }

    cout << "Symbol\tX\tY\tZ" << endl;
    for ( i = 0; i < natoms; i++ ) 
        cout << atoms[i] << endl;

    for ( i = 0; i < 3; i++ ) { // metrictensor: three elements per line
        cout << setprecision(15) 
             << setw(22) << metrictensor[i][0] << " "
             << setw(22) << metrictensor[i][1] << " "
             << setw(22) << metrictensor[i][2] << endl;
    }    

    return 0;
}
